<?php

namespace Litecms\Rentals\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Rentals\Http\Requests\RentalsRequest;
use Litecms\Rentals\Interfaces\RentalsRepositoryInterface;
use Litecms\Rentals\Models\Rentals;

/**
 * Resource controller class for rentals.
 */
class RentalsResourceController extends BaseController
{

    /**
     * Initialize rentals resource controller.
     *
     * @param type RentalsRepositoryInterface $rentals
     *
     * @return null
     */
    public function __construct(RentalsRepositoryInterface $rentals)
    {
        parent::__construct();
        $this->repository = $rentals;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Rentals\Repositories\Criteria\RentalsResourceCriteria::class);
    }

    /**
     * Display a list of rentals.
     *
     * @return Response
     */
    public function index(RentalsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Rentals\Repositories\Presenter\RentalsPresenter::class)
                ->$function();
        }

        $rentals = $this->repository->paginate();

        return $this->response->title(trans('rentals::rentals.names'))
            ->view('rentals::rentals.index', true)
            ->data(compact('rentals'))
            ->output();
    }

    /**
     * Display rentals.
     *
     * @param Request $request
     * @param Model   $rentals
     *
     * @return Response
     */
    public function show(RentalsRequest $request, Rentals $rentals)
    {

        if ($rentals->exists) {
            $view = 'rentals::rentals.show';
        } else {
            $view = 'rentals::rentals.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('rentals::rentals.name'))
            ->data(compact('rentals'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new rentals.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(RentalsRequest $request)
    {

        $rentals = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('rentals::rentals.name')) 
            ->view('rentals::rentals.create', true) 
            ->data(compact('rentals'))
            ->output();
    }

    /**
     * Create new rentals.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RentalsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $rentals                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('rentals::rentals.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('rentals/rentals/' . $rentals->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/rentals/rentals'))
                ->redirect();
        }

    }

    /**
     * Show rentals for editing.
     *
     * @param Request $request
     * @param Model   $rentals
     *
     * @return Response
     */
    public function edit(RentalsRequest $request, Rentals $rentals)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('rentals::rentals.name'))
            ->view('rentals::rentals.edit', true)
            ->data(compact('rentals'))
            ->output();
    }

    /**
     * Update the rentals.
     *
     * @param Request $request
     * @param Model   $rentals
     *
     * @return Response
     */
    public function update(RentalsRequest $request, Rentals $rentals)
    {
        try {
            $attributes = $request->all();

            $rentals->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('rentals::rentals.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('rentals/rentals/' . $rentals->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('rentals/rentals/' . $rentals->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the rentals.
     *
     * @param Model   $rentals
     *
     * @return Response
     */
    public function destroy(RentalsRequest $request, Rentals $rentals)
    {
        try {

            $rentals->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('rentals::rentals.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('rentals/rentals/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('rentals/rentals/' . $rentals->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple rentals.
     *
     * @param Model   $rentals
     *
     * @return Response
     */
    public function delete(RentalsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('rentals::rentals.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('rentals/rentals'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/rentals/rentals'))
                ->redirect();
        }

    }

    /**
     * Restore deleted rentals.
     *
     * @param Model   $rentals
     *
     * @return Response
     */
    public function restore(RentalsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('rentals::rentals.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/rentals/rentals'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/rentals/rentals/'))
                ->redirect();
        }

    }

}
